@@ -1,7 +1,7 @@
 package Bencher;
 
-our $DATE = '2016-01-08'; # DATE
-our $VERSION = '0.26'; # VERSION
+our $DATE = '2016-01-11'; # DATE
+our $VERSION = '0.27'; # VERSION
 
 use 5.010001;
 use strict;
@@ -1288,14 +1288,31 @@ _
         },
 
         on_failure => {
-            summary => "What to do when command fails or Perl code dies",
+            summary => "What to do when there is a failure",
             schema => ['str*', in=>[qw/die skip/]],
             description => <<'_',
 
+For a command participant, failure means non-zero exit code. For a Perl-code
+participant, failure means Perl code dies or (if expected result is specified)
+the result is not equal to the expected result.
+
 The default is "die". When set to "skip", will first run the code of each item
 before benchmarking and trap command failure/Perl exception and if that happens,
 will "skip" the item.
 
+_
+        },
+        on_result_failure => {
+            summary => "What to do when there is a result failure",
+            schema => ['str*', in=>[qw/die skip warn/]],
+            description => <<'_',
+
+This is like `on_failure` except that it specifically refer to the failure of
+item's result not being equal to expected result.
+
+There is an extra choice of `warn` for this type of failure, which is to print a
+warning to STDERR and continue.
+
 _
         },
     },
@@ -1497,6 +1514,7 @@ sub bencher {
         require Time::HiRes;
 
         my $participants = $parsed->{participants};
+        my $datasets = $parsed->{datasets};
         $envres = [200, "OK", [], {}];
 
         my $return_resmeta =
@@ -1538,6 +1556,8 @@ sub bencher {
         }
 
         my $on_failure = $args{on_failure} // $parsed->{on_failure} // 'die';
+        my $on_result_failure = $args{on_result_failure} //
+            $parsed->{on_result_failure} // $on_failure;
         {
             my $fitems = [];
             for my $it (@$items) {
@@ -1550,6 +1570,7 @@ sub bencher {
                         [$it->{code}->()] : $it->{code}->();
                 };
                 my $err = $@;
+
                 if ($err) {
                     if ($on_failure eq 'skip' || $action eq 'show-items-results') {
                         warn "Skipping item #$it->{seq} ($it->{name}) ".
@@ -1559,6 +1580,32 @@ sub bencher {
                         die "Item #$it->{seq} ($it->{name}) fails: $err\n";
                     }
                 }
+
+                $err = "";
+                if (exists($it->{_permute}{dataset}) &&
+                        exists $datasets->[ $it->{_permute}{dataset} ]{result}) {
+                    my $exp_result = $datasets->[ $it->{_permute}{dataset} ]{result};
+                    my $dmp_result = dmp($it->{_result});
+                    my $dmp_exp_result = dmp($exp_result);
+                    if ($dmp_result ne $dmp_exp_result) {
+                        $err = "Result ($dmp_result) is not as expected ($dmp_exp_result)";
+                    }
+                }
+
+                if ($err) {
+                    if ($on_result_failure eq 'skip') {
+                        warn "Skipping item #$it->{seq} ($it->{name}) ".
+                            "due to failure (2): $err\n";
+                        next;
+                    } elsif ($on_result_failure eq 'warn' || $action eq 'show-items-results') {
+                        warn "Warning: item #$it->{seq} ($it->{name}) ".
+                            "has failure (2): $err\n";
+                        next;
+                    } else {
+                        die "Item #$it->{seq} ($it->{name}) fails (2): $err\n";
+                    }
+                }
+
                 push @$fitems, $it;
             }
             $items = $fitems;
@@ -1731,7 +1778,7 @@ Bencher - A benchmark framework
 
 =head1 VERSION
 
-This document describes version 0.26 of Bencher (from Perl distribution Bencher), released on 2016-01-08.
+This document describes version 0.27 of Bencher (from Perl distribution Bencher), released on 2016-01-11.
 
 =head1 SYNOPSIS
 
@@ -1894,12 +1941,26 @@ Benchmark module startup overhead instead of normal benchmark.
 
 =item * B<on_failure> => I<str>
 
-What to do when command fails or Perl code dies.
+What to do when there is a failure.
+
+For a command participant, failure means non-zero exit code. For a Perl-code
+participant, failure means Perl code dies or (if expected result is specified)
+the result is not equal to the expected result.
 
 The default is "die". When set to "skip", will first run the code of each item
 before benchmarking and trap command failure/Perl exception and if that happens,
 will "skip" the item.
 
+=item * B<on_result_failure> => I<str>
+
+What to do when there is a result failure.
+
+This is like C<on_failure> except that it specifically refer to the failure of
+item's result not being equal to expected result.
+
+There is an extra choice of C<warn> for this type of failure, which is to print a
+warning to STDERR and continue.
+
 =item * B<participants> => I<array[hash]>
 
 Add participants.
@@ -2178,12 +2239,28 @@ C<--precision> (CLI).
 
 =item * on_failure (str, "skip"|"die")
 
+For a command participant, failure means non-zero exit code. For a Perl-code
+participant, failure means Perl code dies or (if expected result is specified)
+the result is not equal to the expected result.
+
 The default is "die". When set to "skip", will first run the code of each item
 before benchmarking and trap command failure/Perl exception and if that happens,
 will "skip" the item.
 
 Can be overriden in the CLI with C<--on-failure> option.
 
+=item * on_result_failure (str, "skip"|"die"|"warn")
+
+This is like C<on_failure> except that it specifically refer to the failure of
+item's result not being equal to expected result.
+
+The default is the value of C<on_failure>.
+
+There is an extra choice of `warn` for this type of failure, which is to print a
+warning to STDERR and continue.
+
+Can be overriden in the CLI with C<--on-result-failure> option.
+
 =item * before_parse_scenario (code)
 
 If specified, then this code will be called before parsing scenario. Code will
